/*
* Ademco library
* captainwong
* 2022-6-13 rewrited this C version
*/


#ifndef __ADEMCO_H__
#define __ADEMCO_H__


#pragma once


#ifdef ADEMCO_EXPORTS

# if defined __GNUC__
#  define ADEMCO_EXPORT_SYMBOL __attribute__ ((visibility("default")))
# elif defined(_MSC_VER)
#  define ADEMCO_EXPORT_SYMBOL __declspec(dllexport)
# else
#  define ADEMCO_EXPORT_SYMBOL 
# endif

#else // ADEMCO_EXPORTS

# define ADEMCO_EXPORT_SYMBOL 

#endif


#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <time.h>

#define ADEMCO_OK 0
#define ADEMCO_ERR -1

#define ADEMCO_PACKET_ACCT_MAX_LEN 64 // #acct֮acct󳤶
#define ADEMCO_PACKET_DATA_SEGMENT_EMPTY_LEN 2 // data[]
#define ADEMCO_PACKET_DATA_SEGMENT_FULL_LEN 21 // ǿdataγȣacct6
#define ADEMCO_PACKET_DATA_SEGMENT_FULL_LEN_MAX 64 // ǿdataγȣacctδ֪
#define ADEMCO_PACKET_MAX_LEN 1024
#define CONGWIN_FE100_PACKET_LEN 31 // FE100Э鳤
#define ADEMCO_PACKET_TIMESTAMP_LEN 20
#define ADEMCO_PACKET_XDATA_MAX_LEN 256

// Prototypes
typedef uint32_t AdemcoId;
typedef uint8_t  AdemcoGG;
typedef uint32_t AdemcoZone;

//! IDΧ
#define ADEMCO_ID_MIN 1
#define ADEMCO_ID_MAX 999999 // Կǣ󰲶 ID Ϊ 0x0F423F
#define ADEMCO_ID_SENTINEL (ADEMCO_ID_MAX + 1)
static inline int ademcoIsValidAdemcoId(AdemcoId ademcoId) {
	return ADEMCO_ID_MIN <= ademcoId && ademcoId <= ADEMCO_ID_MAX; 
}

//! Ϊ0ʱʾ
#define ADEMCO_ZONE_FOR_MACHINE_SELF 0
//! ŷΧ
#define ADEMCO_ZONE_MIN 1
//! κͣķ
#define ADEMCO_ZONE_MAX 999
#define ADEMCO_ZONE_SENTINEL (ADEMCO_ZONE_MAX + 1)

//! κͣǷϷ԰0
static inline int ademcoIsValidZone(AdemcoZone zone) {
	return ADEMCO_ZONE_FOR_MACHINE_SELF <= zone && zone <= ADEMCO_ZONE_MAX;
}

//! κͣǷϷ԰0
static inline int ademcoIsValidZoneStrict(AdemcoZone zone) {
	return ADEMCO_ZONE_MIN <= zone && zone <= ADEMCO_ZONE_MAX;
}

#define ADEMCO_GG_MIN 0
#define ADEMCO_GG_MAX 99
#define ADEMCO_GG_SENTINEL 100

// Ademco events
typedef enum AdemcoEvent {
	EVENT_INVALID_EVENT							= 0,

	// -------------------׼Э¼--------------------------------
	// ֻ״̬
	EVENT_ARM									= 3400, // 
	EVENT_DISARM								= 1400, // 
	EVENT_HALFARM_1456							= 1456, // 벼
	EVENT_HALFARM								= 3456, // 벼
	EVENT_EMERGENCY								= 1120, // 

	// 
	EVENT_BURGLAR								= 1130, // 
	EVENT_DOORRINGING							= 1134, // 
	EVENT_FIRE									= 1110, // 
	EVENT_DURESS								= 1121, // в
	EVENT_GAS									= 1151, // ú
	EVENT_WATER									= 1113, // ˮ
	EVENT_TAMPER								= 1137, // 
	EVENT_ZONE_TAMPER							= 1383, // 
	EVENT_BYPASS								= 1570, // ·
	EVENT_BYPASS_RESUME							= 3570, // ·

	// 쳣
	EVENT_AC_BROKE								= 1301, // AC
	EVENT_AC_RECOVER							= 3301, // AC
	EVENT_LOWBATTERY							= 1302, // ͵
	EVENT_BATTERY_RECOVER						= 3302, // 
	EVENT_BADBATTERY							= 1311, // 
	EVENT_BADBATTERY_RECOVER					= 3311, // ָ
	EVENT_SOLARDISTURB							= 1387, // 
	EVENT_SOLARDISTURB_RECOVER					= 3387, // Żָ
	EVENT_DISCONNECT							= 1381, // ʧЧ
	EVENT_RECONNECT								= 3381, // ָ
	EVENT_BATTERY_EXCEPTION						= 1384, // Դ
	EVENT_BATTERY_EXCEPTION_RECOVER				= 3384, // Դϻָ
	EVENT_OTHER_EXCEPTION						= 1380, // 
	EVENT_OTHER_EXCEPTION_RECOVER				= 3380, // ϻָ
	EVENT_LOST									= 1393, // ʧ
	EVENT_LOST_RECOVER							= 3393, // ʧָ

	// ׼¼
	EVENT_3100									= 3100, // Һ쳣ָʾ
	
	// ------------------˽¼-----------------------------------------
	EVENT_SERIAL485DIS							= 1485, // ģ485ӶϿ
	EVENT_SERIAL485CONN							= 3485, // ģ485ӻָ

	EVENT_CONN_HANGUP							= 1700, // ·
	EVENT_CONN_RESUME							= 3700, // ·ָ

	EVENT_DISARM_PWD_ERR						= 1701, // 

	EVENT_SUB_MACHINE_SENSOR_EXCEPTION			= 1702, // ֻ̽ͷ쳣
	EVENT_SUB_MACHINE_SENSOR_RESUME				= 3702, // ֻ̽ͷָ
	EVENT_SUB_MACHINE_POWER_EXCEPTION			= 1703, // ֻԴ쳣
	EVENT_SUB_MACHINE_POWER_RESUME				= 3703, // ֻԴָ

	EVENT_COM_PASSTHROUGH						= 1704, // ͸
	EVENT_ENTER_SET_MODE						= 2704, // ״̬
	EVENT_EXIT_SET_MODE							= 3704,	// ӾķͣΪֹͣҪͣΪܾҪ

	EVENT_QUERY_SUB_MACHINE						= 1705, // ѯֻϢ
	EVENT_WRITE_TO_MACHINE						= 1706, // дϢ

	EVENT_I_AM_NET_MODULE						= 1707, // --ģ
	EVENT_I_AM_GPRS								= 1717, // --GPRS
	EVENT_I_AM_LCD_MACHINE						= 1727, // --Һ
	EVENT_I_AM_WIRE_MACHINE						= 1737, // --
	EVENT_I_AM_WIFI_MACHINE						= 1747, // --WiFi
	EVENT_I_AM_3_SECTION_MACHINE				= 1757, // --
	EVENT_I_AM_IOT_MACHINE						= 1767, // --
	EVENT_I_AM_TRUE_COLOR						= 1777, // --
	EVENT_I_AM_GPRS_IOT							= 1787, // --
	EVENT_I_AM_GPRS_PHONE						= 1797, // --GRPS绰
	EVENT_I_AM_NB_MACHINE						= 1807, // --NB

	EVENT_PHONE_USER_SOS						= 1709, // ֻûSOS
	EVENT_PHONE_USER_CANCLE_ALARM				= 1711, // ֻû

	EVENT_ENTER_SETTING_MODE					= 1712, // ״̬
	EVENT_EXIT_SETTING_MODE						= 3712, // ˳״̬
	EVENT_RESTORE_FACTORY_SETTINGS_710			= 1710, // ָ
	EVENT_RESTORE_FACTORY_SETTINGS				= 1713, // ָ

	// ˴绰ýΪֵ֧ĵ绰ã밢޹
	EVENT_SIM_IS_IOT_CARD						= 1756, // SIMΪ绰
	EVENT_SIM_IS_IOT_PLATFORM_CARD				= 2756, // SIMΪƽ̨绰
	EVENT_SIM_IS_NOT_IOT_CARD					= 3756, // SIMΪ绰

	EVENT_WHAT_IS_YOUR_TYPE						= 1798, // Ҫ
	EVENT_SIGNAL_STRENGTH_CHANGED				= 1799, // źǿȱ仯

	// 202112417:06:55޸ģԱ3B0 3B2
	EVENT_OFFLINE								= 1944, // 
	EVENT_ONLINE								= 1946, // 

#define ADEMCO_EVENT_SENTINEL					10000

}AdemcoEvent;

static const AdemcoEvent AdemcoEvents[] = {
	EVENT_ARM,
	EVENT_DISARM,
	EVENT_HALFARM,
	EVENT_EMERGENCY,

	EVENT_BURGLAR,
	EVENT_DOORRINGING,
	EVENT_FIRE,
	EVENT_DURESS,
	EVENT_GAS,
	EVENT_WATER,
	EVENT_TAMPER,
	EVENT_ZONE_TAMPER,
	EVENT_BYPASS,
	EVENT_BYPASS_RESUME,

	EVENT_AC_BROKE,
	EVENT_AC_RECOVER,
	EVENT_LOWBATTERY,
	EVENT_BATTERY_RECOVER,
	EVENT_BADBATTERY,
	EVENT_BADBATTERY_RECOVER,
	EVENT_SOLARDISTURB,
	EVENT_SOLARDISTURB_RECOVER,
	EVENT_DISCONNECT,
	EVENT_RECONNECT,
	EVENT_LOST,
	EVENT_LOST_RECOVER,
	EVENT_3100,
	EVENT_BATTERY_EXCEPTION,
	EVENT_BATTERY_EXCEPTION_RECOVER,
	EVENT_OTHER_EXCEPTION,
	EVENT_OTHER_EXCEPTION_RECOVER,

	EVENT_SERIAL485DIS,
	EVENT_SERIAL485CONN,
	EVENT_CONN_HANGUP,
	EVENT_CONN_RESUME,
	EVENT_DISARM_PWD_ERR,

	EVENT_SUB_MACHINE_SENSOR_EXCEPTION,
	EVENT_SUB_MACHINE_SENSOR_RESUME,
	EVENT_SUB_MACHINE_POWER_EXCEPTION,
	EVENT_SUB_MACHINE_POWER_RESUME,
	EVENT_COM_PASSTHROUGH,

	EVENT_ENTER_SET_MODE,
	EVENT_EXIT_SET_MODE,
	EVENT_QUERY_SUB_MACHINE,
	EVENT_WRITE_TO_MACHINE,

	EVENT_I_AM_NET_MODULE,
	EVENT_I_AM_GPRS,
	EVENT_I_AM_LCD_MACHINE,
	EVENT_I_AM_WIRE_MACHINE,
	EVENT_I_AM_WIFI_MACHINE,
	EVENT_I_AM_3_SECTION_MACHINE,
	EVENT_I_AM_IOT_MACHINE,
	EVENT_I_AM_TRUE_COLOR,
	EVENT_I_AM_GPRS_IOT,
	EVENT_I_AM_GPRS_PHONE,
	EVENT_I_AM_NB_MACHINE,

	EVENT_SIM_IS_IOT_CARD,
	EVENT_SIM_IS_IOT_PLATFORM_CARD,
	EVENT_SIM_IS_NOT_IOT_CARD,

	EVENT_PHONE_USER_SOS,
	EVENT_PHONE_USER_CANCLE_ALARM,
	EVENT_ENTER_SETTING_MODE,
	EVENT_EXIT_SETTING_MODE,
	EVENT_RESTORE_FACTORY_SETTINGS,

	EVENT_WHAT_IS_YOUR_TYPE,
	EVENT_SIGNAL_STRENGTH_CHANGED,
};

// ¼
typedef enum AdemcoEventLevel
{
	EVENT_LEVEL_NULL,
	EVENT_LEVEL_STATUS,				// ״̬
	EVENT_LEVEL_EXCEPTION_RESUME,	// ɫ
	EVENT_LEVEL_EXCEPTION,			// ɫ
	EVENT_LEVEL_ALARM,				// ɫ
}AdemcoEventLevel;

// Э
typedef enum AdemcoParseResult {
	RESULT_OK, //! ȷһ
	RESULT_NOT_ENOUGH, //! ȷһҪݣ´μ
	RESULT_ERROR, //! ݣӦݰ
}AdemcoParseResult;

// Эdata
typedef struct AdemcoDataSegment {
	uint8_t raw[ADEMCO_PACKET_DATA_SEGMENT_FULL_LEN_MAX];
	int raw_len;
	AdemcoId ademcoId;
	AdemcoEvent ademcoEvent;
	AdemcoGG gg;
	AdemcoZone zone;
}AdemcoDataSegment;

typedef enum AdemcoXDataLengthFormat {
	TWO_HEX, // hexֽڱʾ
	FOUR_DECIMAL, // ĸdecimal char ʾ
}AdemcoXDataLengthFormat;

typedef struct AdemcoXDataSegment {
	AdemcoXDataLengthFormat lenghFormat;
	uint8_t raw[ADEMCO_PACKET_XDATA_MAX_LEN];
	int raw_len;
}AdemcoXDataSegment;

#define ADEMCO_PACKET_ID_NULL	 "\"NULL\""
#define ADEMCO_PACKET_ID_ACK	 "\"ACK\""
#define ADEMCO_PACKET_ID_NAK     "\"NAK\""
#define ADEMCO_PACKET_ID_DUH     "\"DUH\""
#define ADEMCO_PACKET_ID_HB      "\"HENG-BO\""
#define ADEMCO_PACKET_ID_ADM_CID "\"ADM-CID\""
#define ADEMCO_PACKET_ID_MOD_REG "\"MODULE_REG\""
#define ADEMCO_PACKET_ID_REG_RSP "\"REG-RSP\""

typedef enum AdemcoPacketId {
	AID_NULL,
	AID_ACK,
	AID_NAK,
	AID_DUH,
	AID_HB,
	AID_ADM_CID,
	AID_MOD_REG,
	AID_REG_RSP,
	AID_COUNT,
	AID_INVALID = -1,
}AdemcoPacketId;

#define ADEMCO_RRCVR_DEFAULT "R123ABC"
#define ADEMCO_LPREF_DEFAULT "L456DEF"

#define ADEMCO_PACKET_PREFIX '\n'
#define ADEMCO_PACKET_SUFIX  '\r'

typedef struct AdemcoPacket {
	uint16_t crc;
	int len;
	AdemcoPacketId id;
	uint16_t seq;
	char acct[ADEMCO_PACKET_ACCT_MAX_LEN + 1];
	AdemcoDataSegment data;
	AdemcoXDataSegment xdata;
	time_t timestamp;
}AdemcoPacket;


// Functions

//! Ƿ״̬¼
ADEMCO_EXPORT_SYMBOL int ademcoIsMachineStatusEvent(AdemcoEvent ademcoEvent);
//! Ƿ¼
ADEMCO_EXPORT_SYMBOL int ademcoIsMachineTypeEvent(AdemcoEvent ademcoEvent);
//! ¼ǷҪԴ
ADEMCO_EXPORT_SYMBOL int ademcoIsEventNeedControlSource(AdemcoEvent ademcoEvent);
//! ȡ¼
ADEMCO_EXPORT_SYMBOL AdemcoEventLevel ademcoGetEventLevel(AdemcoEvent ademcoEvent);
//! ȡ쳣ָ¼Ӧ쳣¼
ADEMCO_EXPORT_SYMBOL AdemcoEvent ademcoGetExceptionEventByResumeEvent(AdemcoEvent resumeEvent);
ADEMCO_EXPORT_SYMBOL const char* ademcoEventToString(AdemcoEvent ademcoEvent);
ADEMCO_EXPORT_SYMBOL const wchar_t* ademcoEventToStringChinese(AdemcoEvent ademcoEvent);

ADEMCO_EXPORT_SYMBOL int ademcoAppendDataSegment(uint8_t* packet, AdemcoId ademcoId, AdemcoEvent ademcoEvent, AdemcoGG gg, AdemcoZone zone);
ADEMCO_EXPORT_SYMBOL int ademcoAppendDataSegment2(uint8_t* packet, const AdemcoDataSegment* dataSegment);
ADEMCO_EXPORT_SYMBOL AdemcoParseResult ademcoParseDataSegment(const uint8_t* packet, int packet_len, AdemcoDataSegment* dataSegment);
// return 0 for empty packet, CONGWIN_FE100_PACKET_LEN for non-empty packet
ADEMCO_EXPORT_SYMBOL int ademcoDataSegmentToCongwinFe100(uint8_t* fe100, const AdemcoDataSegment* dataSegment);
ADEMCO_EXPORT_SYMBOL const uint8_t* ademcoXDataGetValidContentAddr(const AdemcoXDataSegment* xdata);
ADEMCO_EXPORT_SYMBOL int ademcoXDataGetValidContentLen(const AdemcoXDataSegment* xdata);
// return ADEMCO_OK for success, return ADEMCO_ERR for len is too long
ADEMCO_EXPORT_SYMBOL int ademcoMakeXData(AdemcoXDataSegment* xdata, AdemcoXDataLengthFormat xlf, const uint8_t* content, int len);

ADEMCO_EXPORT_SYMBOL int isAdemcoId(const char* standard, const char* id, int len);
ADEMCO_EXPORT_SYMBOL AdemcoPacketId getAdemcoId(const char* id, int len);

/*
* ademcoMake*Packet functions
* call with buff = NULL or len < length needed, will return length needed.
* if buff != NULL and len >= length needed, return length needed and copy data to buff
* otherwise return 0 for fail
*/

ADEMCO_EXPORT_SYMBOL int ademcoMakeNullPacket(uint8_t* buff, int len, uint16_t seq, const char* acct, AdemcoId ademcoId);
ADEMCO_EXPORT_SYMBOL int ademcoMakeAckPacket(uint8_t* buff, int len, uint16_t seq, const char* acct, AdemcoId ademcoId);
ADEMCO_EXPORT_SYMBOL int ademcoMakeNakPacket(uint8_t* buff, int len, uint16_t seq, const char* acct, AdemcoId ademcoId);
ADEMCO_EXPORT_SYMBOL int ademcoMakeHbPacket(uint8_t* buff, int len, uint16_t seq, const char* acct, AdemcoId ademcoId, 
											AdemcoEvent ademcoEvent, AdemcoGG gg, AdemcoZone zone, AdemcoXDataSegment* xdata);

ADEMCO_EXPORT_SYMBOL AdemcoParseResult ademcoPacketParse(const uint8_t* buff, int len, AdemcoPacket* pkt, int* cb_commited);

ADEMCO_EXPORT_SYMBOL uint16_t ademcoCRC8(char c, uint16_t crc);
ADEMCO_EXPORT_SYMBOL uint16_t ademcoCRC16(const uint8_t* buff, int len, uint16_t crc);


/* Hengbo */

//! ״̬GGΧ 1~3
#define HB_3SECTION_MACHINE_GG_MIN 1
#define HB_3SECTION_MACHINE_GG_MAX 3

//! ǷϷ״̬GG
static inline int hbIsValid3SectionMachineGG(AdemcoGG gg) {
	return HB_3SECTION_MACHINE_GG_MIN <= gg && gg <= HB_3SECTION_MACHINE_GG_MAX;
}

typedef enum HbMachineStatus {
	HMS_ARM,
	HMS_HALF_ARM,
	HMS_DISARM,
	HMS_SETTING,
	HMS_COUNT,
	HMS_INVALID = -1,
}HbMachineStatus;

typedef enum HbMachineType {
	//! 0 WiFi ͣã
	HMT_WIFI,
	//! 1  δã
	HMT_CAMERA,
	//! 2 򻯵 [ HB-5050G-4GW ]
	HMT_GPRS_IOT,
	//! 3 ģ [ HB-G250 ]
	HMT_NETMOD,
	//! 4 Ľʽ [ HB-4040G HB-5050G HB-5050-4G ]
	HMT_GPRS,
	//! 5 Һ [ HB-BJQ560 HB-BJQ-560B ]
	HMT_LCD,
	//! 6  [ HB-4040R HB-5050R ]
	HMT_WIRED,
	//! 7  [ HTZ-G1000 HTZ-G1000-4G ]
	HMT_TRUE_COLOR,
	//! 8  [ HTZ-G1000 HTZ-G1000-4G ]
	HMT_3_SECTION,
	//! 9  [ HB-2050-4GW ]
	HMT_IOT,
	//! 10 GPRSԴ绰 [ HB-2050 ]
	HMT_GPRS_PHONE,
	//! 11 NB
	HMT_NB,

	HMT_COUNT,
	HMT_INVALID = -1,
}HbMachineType;

typedef enum HbZoneProperty {
	//! ˾ȫ
	HZP_BUGLAR,
	//! ˾
	HZP_EMERGENCY,
	//! 𾯷
	HZP_FIRE,
	//! вȷ
	HZP_DURESS,
	//! ȼ
	HZP_GAS,
	//! ˮ
	HZP_WATER,
	//! ֻ
	HZP_SUB_MACHINE,
	//! ң
	HZP_REMOTE_CONTROL,
	//! ˾
	HZP_BUGLAR_HALF,
	//! η
	HZP_SHIELD,
	//! 
	HZP_DOOR_RING,
	//! 
	HZP_RESERVED_0B,
	//! 
	HZP_RESERVED_0C,
	//! 
	HZP_RESERVED_0D,
	//! 
	HZP_RESERVED_0E,
	//! ·
	//! 20199216:01:58 𽨷
	HZP_BYPASS,

	HZP_COUNT,
	HZP_INVALID = -1,
}HbZoneProperty;

typedef struct HbMachineTimePoint {
	uint8_t hour;
	uint8_t minute;
}HbMachineTimePoint;

typedef struct HbTimer {
	HbMachineTimePoint armAt;
	HbMachineTimePoint disarmAt;
}HbTimer;

typedef union HbMachineTimer {
	HbTimer timer[2]; // 2 set of timer
	uint8_t data[8];
}HbMachineTimer;

// ͸󳤶
#define HB_COM_DATA_MAX_LEN 32

// 
typedef enum HbComRequestType {
	HbComReq_A0, // Ҫ״̬
	HbComReq_A1, // Ҫ
	HbComReq_A2, // Ҫ
	HbComReq_A3, // ޸ķ
	HbComReq_A5, // ȡʱ
	HbComReq_A7, // öʱ
	HbComReq_A9, // TODO
	HbComReq_AA, // ޸ķ̽ͷʧ/ʧ
	HbComReq_AC, // Ҫ̽ͷʧ/ʧ--һҪ
	HbComReq_AD, // Ҫ̽ͷʧ/ʧ--Ҫ
	HbComReq_AE, // 
	HbComReq_B0, // Ҫ״̬
	HbComReq_RD_acct, // ȡ˺
	HbComReq_WR_acct, // д˺
	HbComReq_Invalid = -1,
}HbComRequestType;

// Ӧ
typedef enum HbComResponseType {
	HbComResp_A0, // Ҫ״̬Ӧ
	HbComResp_A2, // ҪӦ
	HbComResp_A3, // ѧ뿪ʼӦ
	HbComResp_A4, // ޸ķӦ
	HbComResp_A6, // ȡʱӦ
	HbComResp_A7, // ʱӦ
	HbComResp_A8, // ܾûʧܻӦ
	HbComResp_A9, // TODO
	HbComResp_AB, // ޸ķ̽ͷʧ/ʧӦ
	HbComResp_AD, // Ҫ̽ͷʧ/ʧӦ
	HbComResp_AF, // Ӧ
	HbComResp_B1, // ״̬Ӧ
	HbComResp_Invalid = -1,
}HbComResponseType;

//! Ҫ״̬ EB AB 3F A0 75
#define HbComReq_A0_len 5
#define HbComReq_A0_data "\xEB\xAB\x3F\xA0\x75"

//! Ӧ״̬ EB BA 3F 07 P0 A0 P1 P2 P3 SUM
#define HbComResp_A0_len 9
#define HbComResp_A0_head "\xEB\xBA\x3F\x07"

//! Ҫ EB AB 3F A1 76
#define HbComReq_A1_len 5
#define HbComReq_A1_data "\xEB\xAB\x3F\xA1\x76"

//! Ӧ EB BA 3F PN P0 A2 [Z, P]xN P1 SUM
#define HbComResp_A2_len_min 8 // ޷ʱСΪ8
#define HbComResp_A2_max_zone 20 // ԰ 20 
#define HbComResp_A2_len_max (HbComResp_A2_len_min + HbComResp_A2_max_zone * 2) // һ8+20*2=48ֽ
#define HbComResp_A2_nomore 0xFF // P1 ޸
#define HbComResp_A2_hasmore 0x00 // P1 и

//! Ҫ EB AB 3F A2 77
//! ӦյZoneResponseparam0xFFʱͣҪȫ
#define HbComReq_A2_len 5
#define HbComReq_A2_data "\xEB\xAB\x3F\xA2\x77"

//! ޸ EB CB 3F 09 A3 P1 P2 P3 SUM
#define HbComReq_A3_len 9
#define HbComReq_A3_head "\xEB\xCB\x3F\x09\xA3"
#define HbComReq_A3_op_del		0x00 //! ɾ
#define HbComReq_A3_op_learn	0x01 //! ѧ
#define HbComReq_A3_op_stop		0x02 //! ֹͣѧ
#define HbComReq_A3_op_modify	0x04 //! ޸ķ

//! ѧ뿪ʼӦ EB BA 3F 07 P0 A3 5A
//! ΪѧʱҪȴⲿźţû̽ȻӦA3ʾѾʼѧ룬ѧɹʱ ZoneOpResponse A4
#define HbComResp_A3_len 7
#define HbComResp_A3_data "\xEB\xBA\x3F\x07\xCC\xA3\x5A"

//! ޸ķӦ EB BA 3F 0A P0 A4 P1 P2 P3 SUM
#define HbComResp_A4_len 10
#define HbComResp_A4_head "\xEB\xBA\x3F\x0A\xCC\xA4"
#define HbComResp_A4_fail	0x00 //! ʧ
#define HbComResp_A4_ok		0x01 //! ɹ
#define HbComResp_A4_dup	0x02 //! ʧ--
#define HbComResp_A4_ne		0x03 //! ʧ--δ not exists

//! ȡʱ EB AB 3F A5 7A
#define HbComReq_A5_len 5
#define HbComReq_A5_data "\xEB\xAB\x3F\xA5\x7A"

//! ȡʱӦ EB BA 3F 0F P0 A6 H1 M1 H2 M2 H3 M3 H4 M4 SUM
#define HbComResp_A6_len 15
#define HbComResp_A6_head "\xEB\xBA\x3F\x0F\xCC\xA6"

//! ʱ EB CB 3F 0E A7 H1 M1 H2 M2 H3 M3 H4 M4 SUM
#define HbComReq_A7_len 14
#define HbComReq_A7_head "\xEB\xCB\x3F\x0E\xA7"

//! ʱӦ EB BA 3F 07 P0 A7 SUM
#define HbComResp_A7_len 7
#define HbComResp_A7_data "\xEB\xBA\x3F\x07\xCC\xA7\x5E"

//! ܾûӦ EB BA 3F 07 P0 A8 SUM
//! κβΪǷA8ֱӻظ
#define HbComResp_A8_len 7
#define HbComResp_A8_data "\xEB\xBA\x3F\x07\xCC\xA8\x5F"

//! ޸ķ̽ͷʧ/ʧ EB CB 3F 08 AA P1 P2 SUM
#define HbComReq_AA_len 8
#define HbComReq_AA_head "\xEB\xAB\x3F\x08\xAA"

//! ޸ķ̽ͷʧ/ʧӦ EB BA 3F 09 P0 AB P1 P2 SUM
#define HbComResp_AB_len 9
#define HbComResp_AB_head "\xEB\xBA\x3F\x09\xCC\xAB"

//! Ҫ̽ͷʧ/ʧ  һҪ EB AB 3F AC 81
#define HbComReq_AC_len 5
#define HbComReq_AC_data "\xEB\xAB\x3F\xAC\x81"

//! Ҫ̽ͷʧ/ʧ  Ҫ EB AB 3F AD 82
#define HbComReq_AD_len 5
#define HbComReq_AD_data "\xEB\xAB\x3F\xAD\x82"

//! Ҫ̽ͷʧ/ʧӦ EB BA 3F PN P0 AD P1 DATA P2 SUM
#define HbComResp_AD_len_min 9 // ޷ʧʱһСΪ9
#define HbComResp_AD_max_zone 20 // һ20
#define HbComResp_AD_len_max (HbComResp_AD_len_min + HbComResp_AD_max_zone * 2) // һ 9 + 20 *2 = 49 ֽ
#define HbComResp_AD_p1_single 0xF0 // Եֽڱʾ
#define HbComResp_AD_p1_double 0xF1 // ˫ֽڱʾ
#define HbComResp_AD_head "\xEB\xBA\x3F\x09\xCC\xAD"

#define HbCom_3sec_disarm	0x00 // ״̬
#define HbCom_3sec_arm		0x01 // ״̬

//!  EB CB 3F 08 AE P1 P2 SUM
#define HbComReq_AE_len 8
#define HbComReq_AE_head "\xEB\xCB\x3F\x08\xAE"

//! Ӧ EB BA 3F 08 P0 AF P1 P2 SUM
#define HbComResp_AF_len 9
#define HbComResp_AF_head "\xEB\xBA\x3F\x08\xCC\xAF"

//! Ҫ״̬ EB CB 3F 06 B0 AB
#define HbComReq_B0_len 6
#define HbComReq_B0_data "\xEB\xCB\x3F\x06\xB0\xAB"

//! ״̬Ӧ EB BA 3F 08 P0 B1 P1 SUM
#define HbComResp_B1_len 8
#define HbComResp_B1_head "\xEB\xBA\x3F\x08"

//! ȡ˺ 
#define HbComReq_RD_acct_len 7
#define HbComReq_RD_acct_data "\xEB\xBA\x3F\x07\x00\x4C\x37"

//! ȡ˺ŻӦ
#define HbComResp_RD_acct_len 15
#define HbComResp_RD_acct_head "\xEB\xCB\x3F\x0F\x4C"

//! д˺
#define HbComReq_WR_acct_len 15
#define HbComReq_WR_acct_head "\xEB\xCB\x3F\x0F\x4D"

//! д˺ŻӦȡ˺ͬ
#define HbComResp_WR_acct_len HbComReq_RD_acct_len
#define HbComResp_WR_acct_head HbComReq_WR_acct_head

static const HbZoneProperty hbZoneProperties[12] = {
	HZP_BUGLAR, HZP_EMERGENCY, HZP_FIRE, HZP_DURESS, HZP_GAS, HZP_WATER, HZP_SUB_MACHINE, 
	HZP_REMOTE_CONTROL, HZP_BUGLAR_HALF, HZP_SHIELD, HZP_DOOR_RING, HZP_BYPASS
};

// return available count
ADEMCO_EXPORT_SYMBOL int hbGetAvailableZoneProperties(HbMachineType type, HbZoneProperty props[12]);

//! ŸͺŲͬͬ
ADEMCO_EXPORT_SYMBOL uint16_t hbZoneMax(HbMachineType type);
//! ǷϷ԰0
ADEMCO_EXPORT_SYMBOL int hbIsValidZone(HbMachineType type, uint16_t zone);
//! ǷϷԿ԰0
ADEMCO_EXPORT_SYMBOL int hbIsValidZoneStrict(HbMachineType type, uint16_t zone);
//! ǷͶʹ
ADEMCO_EXPORT_SYMBOL int hbMachineIsSelling(HbMachineType type);
//! Ƿв
ADEMCO_EXPORT_SYMBOL int hbMachineCanArm(HbMachineType type);
//! Ƿг
ADEMCO_EXPORT_SYMBOL int hbMachineCanDisarm(HbMachineType type);
//! ǷԽ״̬
ADEMCO_EXPORT_SYMBOL int hbMachineCanEnterSettings(HbMachineType type);
//! Ƿа벼
ADEMCO_EXPORT_SYMBOL int hbMachineCanHalfArm(HbMachineType type);
//! ǷԱźǿ
ADEMCO_EXPORT_SYMBOL int hbMachineCanReportSignalStrength(HbMachineType type);
//! ǷԶűͨ簢ȣ
ADEMCO_EXPORT_SYMBOL int hbMachineCanReportBySMS(HbMachineType type);
//! Ƿ֧߷
ADEMCO_EXPORT_SYMBOL int hbMachineHasWiredZone(HbMachineType type);
//! С߷
ADEMCO_EXPORT_SYMBOL AdemcoZone hbWiredZoneMin(HbMachineType type);
//! ߷
ADEMCO_EXPORT_SYMBOL AdemcoZone hbWiredZoneMax(HbMachineType type);
//! Ƿֱдݣ룩
ADEMCO_EXPORT_SYMBOL int hbMachineCanDirectlyWriteZone(HbMachineType type);
//! ǷԹطֻ
ADEMCO_EXPORT_SYMBOL int hbMachineCanLinkSubMachine(HbMachineType type);
//! ݷжǷ֧ʧ
//! ʧʵֵĸûϵΪֻԵĲſ
ADEMCO_EXPORT_SYMBOL int hbZonePropCanReportLost(HbZoneProperty zp);
ADEMCO_EXPORT_SYMBOL void hbInitMachineTimer(HbMachineTimer* timer);
ADEMCO_EXPORT_SYMBOL int hbIsValidTimer(HbTimer* timer);
ADEMCO_EXPORT_SYMBOL AdemcoEvent hbMachineStatusToAdemcoEvent(HbMachineStatus status);
ADEMCO_EXPORT_SYMBOL HbMachineStatus hbMachineStatusFromAdemcoEvent(AdemcoEvent ademcoEvent);
ADEMCO_EXPORT_SYMBOL AdemcoEvent hbMachineTypeToAdemcoEvent(HbMachineType type);
ADEMCO_EXPORT_SYMBOL HbMachineType hbMachineTypeFromAdemcoEvent(AdemcoEvent ademcoEvent);
ADEMCO_EXPORT_SYMBOL AdemcoEvent hbZonePropertyToAdemcoEvent(HbZoneProperty zp);
ADEMCO_EXPORT_SYMBOL const char* hbMachineTypeToString(HbMachineType type);
ADEMCO_EXPORT_SYMBOL const wchar_t* hbMachineTypeToStringChinese(HbMachineType type);
ADEMCO_EXPORT_SYMBOL const char* hbZonePropertyToString(HbZoneProperty zp);
ADEMCO_EXPORT_SYMBOL const wchar_t* hbZonePropertyToStringChinese(HbZoneProperty zp);
ADEMCO_EXPORT_SYMBOL const char* hbGetZoneFormatString(HbMachineType type);
ADEMCO_EXPORT_SYMBOL const wchar_t* hbGetZoneFormatStringW(HbMachineType type);
// ۼУ飬data[0] ~ data[len-2]Уͷdata[len-1]
ADEMCO_EXPORT_SYMBOL void hbSum(uint8_t* data, int len);
// УǷȷ, return 0 for incorrect, otherwise correct
ADEMCO_EXPORT_SYMBOL int hbCheckSum(const uint8_t* data, int len);
ADEMCO_EXPORT_SYMBOL HbComRequestType hbComParseRequest(const uint8_t* buff, int len);
ADEMCO_EXPORT_SYMBOL HbComResponseType hbParseComResponse(const uint8_t* buff, int len);

// һԸߵֽڱʾʮתΪ10ַ0xf'0'~'9'ַַֹͣ
// ʾ飺0x18 0x24 0x08 0x88 0x10 0x1f, 0xffַ"18240888101"
ADEMCO_EXPORT_SYMBOL int hbHiLoArrayToDecStr(char* str, const uint8_t* arr, int len);
// һ10ַתΪߵֽڱʾ飬ʡռ䣬鳤
ADEMCO_EXPORT_SYMBOL int hbDecStrToHiLoArray(uint8_t* arr, const char* str, int len);


#ifdef __cplusplus
}
#endif

#endif // __ADEMCO_H__
